import { ConsoleTypes, ConsoleLevels } from '../models/index';
import { _replace, _recover, _log } from '../tools/helpers';
import BasicObserverClass from './index';
import { OBSERVER_DEFAULT_OPTIONS } from '../constants';
export default class ConsoleObserverClass extends BasicObserverClass {
    constructor(options) {
        super();
        this.name = 'ConsoleObserverClass';
        this.consoleLevels = Object.keys(ConsoleLevels);
        this.options = {
            info: true,
            error: true,
            log: false,
            warn: true,
            debug: false
        };
        this.status = OBSERVER_DEFAULT_OPTIONS.console;
        if (options === false)
            return;
        if (typeof options === 'object') {
            this.options = { ...this.options, ...options };
        }
    }
    install() {
        const { status, $emit } = this;
        this.consoleLevels.forEach((level) => {
            if (!this.options[level])
                return;
            function consoleReplacement(originalConsoleFunc) {
                return function (...args) {
                    if (!args.length)
                        return;
                    const record = {
                        type: ConsoleTypes.console,
                        l: level,
                        msg: args
                    };
                    $emit('observed', record);
                    if (originalConsoleFunc) {
                        originalConsoleFunc.call(console, ...args);
                    }
                };
            }
            _replace(console, level, consoleReplacement);
            status[level] = true;
        });
        _log('console observer ready!');
    }
    uninstall() {
        this.consoleLevels.forEach((level) => {
            if (!this.options[level])
                return;
            _recover(console, level);
            status[level] = false;
        });
    }
}
