import { _recover, _log, _parseURL, _replace } from '../tools/helpers';
import { HistoryTypes } from '../models/index';
import BasicObserverClass from './index';
export default class HistoryObserverClass extends BasicObserverClass {
    constructor(options) {
        super();
        this.name = 'HistoryObserverClass';
        this.status = false;
        if (options === false)
            return;
    }
    getHistoryRecord(from, to) {
        const parsedHref = _parseURL(location.href);
        const parsedTo = _parseURL(to);
        let parsedFrom = _parseURL(from);
        if (!parsedFrom.path) {
            parsedFrom = parsedHref;
        }
        this.lastHref = to;
        const record = {
            type: HistoryTypes.history,
            from: parsedFrom.relative,
            to: parsedTo.relative
        };
        const { $emit } = this;
        $emit('observed', record);
    }
    isSupportHistory() {
        return ('history' in window &&
            !!window.history.pushState &&
            !!window.history.replaceState);
    }
    install() {
        if (!this.isSupportHistory())
            return;
        const { getHistoryRecord } = this;
        const self = this;
        _replace(window, 'onpopstate', function (originalHandler) {
            return function (...args) {
                getHistoryRecord.call(self, self.lastHref, location.href);
                originalHandler && originalHandler.apply(this, args);
            };
        });
        function historyReplacement(originalMethod) {
            return function (...args) {
                const url = args.length > 2 ? args[2] : undefined;
                if (url)
                    getHistoryRecord.call(self, self.lastHref, String(url));
                return originalMethod.apply(this, args);
            };
        }
        _replace(window.history, 'pushState', historyReplacement);
        _replace(window.history, 'replaceState', historyReplacement);
        _log('history installed');
        this.status = true;
    }
    uninstall() {
        _recover(window, 'onpopstate');
        _recover(window.history, 'pushState');
        _recover(window.history, 'replaceState');
        this.status = false;
    }
}
