import { isFunction } from '../tools/is';
export default class BasicObserverClass {
    constructor() {
        this.queues = new Map();
        this.$on = (hook, action) => {
            const { queues } = this;
            const existingTasks = queues.get(hook) || [];
            queues.set(hook, [...existingTasks, action]);
        };
        this.$off = (hook, thisAction) => {
            const Q = this.queues.get(hook) || [];
            if (!Q.length) {
                return;
            }
            const index = Q.indexOf(thisAction);
            if (index !== -1) {
                Q.splice(index, 1);
                this.queues.set(hook, Q);
            }
        };
        this.$emit = (hook, ...args) => {
            const Q = this.queues.get(hook) || [];
            if (!Q.length) {
                return;
            }
            try {
                Q.forEach(action => {
                    if (isFunction(action)) {
                        action(...args);
                    }
                });
            }
            catch (error) {
                console.error(error);
            }
        };
    }
}
