import { ErrorTypes } from '../models/index';
import { _replace, _log, _recover } from '../tools/helpers';
import BasicObserverClass from './index';
import { OBSERVER_DEFAULT_OPTIONS } from '../constants';
export default class JSErrorObserverClass extends BasicObserverClass {
    constructor(options) {
        super();
        this.name = 'JSErrorObserverClass';
        this.options = OBSERVER_DEFAULT_OPTIONS.error;
        this.status = {
            jserror: false,
            unhandledrejection: false
        };
        this.getGlobalerrorReocrd = (errevt) => {
            const { message: msg, lineno, colno, error: err, filename: url } = errevt;
            const record = {
                type: ErrorTypes.jserr,
                url,
                line: `${lineno}:${colno}`,
                msg,
                err
            };
            const { $emit } = this;
            $emit('observed', record, errevt);
        };
        this.getUnhandlerejectionRecord = (errevt) => {
            let _errevt = { ...errevt };
            if (!_errevt) {
                _errevt.reason = 'undefined';
            }
            const record = {
                type: ErrorTypes.unhandledrejection,
                msg: _errevt.reason
            };
            const { $emit } = this;
            $emit('observed', record, errevt);
        };
        if (options === false)
            return;
        if (typeof options === 'object') {
            this.options = { ...this.options, ...options };
        }
    }
    getStackTeace() {
    }
    installGlobalerrorHandler() {
        const { getGlobalerrorReocrd } = this;
        _replace(window, 'onerror', oldOnerrorHandler => {
            const recorderOnerrorHandler = function (message, filename, lineno, colno, error) {
                if (error && error instanceof ErrorEvent) {
                    getGlobalerrorReocrd(error);
                }
                else if (message instanceof ErrorEvent) {
                    getGlobalerrorReocrd(message);
                }
                else {
                    getGlobalerrorReocrd({
                        message,
                        filename,
                        lineno,
                        colno,
                        error
                    });
                }
                if (oldOnerrorHandler) {
                    oldOnerrorHandler.apply(window, arguments);
                }
            };
            return recorderOnerrorHandler;
        });
    }
    installUnhanldledrejectionHandler() {
        const { getUnhandlerejectionRecord } = this;
        _replace(window, 'onunhandledrejection', originalUnhanldledrejectionHandler => {
            return function (errevt) {
                getUnhandlerejectionRecord(errevt);
                if (originalUnhanldledrejectionHandler) {
                    originalUnhanldledrejectionHandler.call(this, errevt);
                }
            };
        });
    }
    install() {
        const { jserror, unhandledrejection } = this.options;
        if (jserror) {
            this.installGlobalerrorHandler();
            this.status.jserror = true;
            Object.defineProperty(window, 'onerror', {
                set(newHook) {
                    if (!newHook.__recorder__) {
                        _log('recorder error handler died!');
                    }
                }
            });
        }
        if (unhandledrejection) {
            this.installUnhanldledrejectionHandler();
            this.status.unhandledrejection = true;
        }
        _log('error observer ready!');
    }
    uninstall() {
        const { jserror, unhandledrejection } = this.options;
        if (jserror) {
            _recover(window, 'onerror');
            this.status.jserror = false;
        }
        if (unhandledrejection) {
            _recover(window, 'onunhandledrejection');
            this.status.unhandledrejection = false;
        }
    }
}
