import { MouseTypes } from '../models/index';
import { _throttle, _log } from '../tools/helpers';
import BasicObserverClass from './index';
import { OBSERVER_DEFAULT_OPTIONS } from '../constants';
export default class MouseObserverClass extends BasicObserverClass {
    constructor(options) {
        super();
        this.name = 'MouseObserverClass';
        this.listeners = [];
        this.options = OBSERVER_DEFAULT_OPTIONS.mouse;
        this.status = {
            click: false,
            mousemove: false
        };
        this.addListener = ({ target, event, callback, options = false }, cb) => {
            target.addEventListener(event, callback, options);
            this.listeners.push({
                target,
                event,
                callback
            });
            cb && cb();
        };
        this.getMouseClickRecord = (evt) => {
            const { pageX: x, pageY: y } = evt;
            const record = { type: MouseTypes.click, x, y };
            const { $emit } = this;
            $emit('observed', record);
        };
        this.getMouseMoveRecord = (evt) => {
            const { pageX: x, pageY: y } = evt;
            const record = { type: MouseTypes.move, x, y };
            const { $emit } = this;
            $emit('observed', record);
        };
        if (options === false)
            return;
        if (typeof options === 'object') {
            this.options = { ...this.options, ...options };
        }
    }
    install() {
        const { addListener } = this;
        const { click, mousemove } = this.options;
        if (click) {
            addListener({
                target: document,
                event: 'click',
                callback: this.getMouseClickRecord
            });
            this.status.click = true;
        }
        if (mousemove) {
            addListener({
                target: document,
                event: 'mousemove',
                callback: _throttle(this.getMouseMoveRecord, 100)
            });
            this.status.mousemove = true;
        }
        _log('mouse observer ready!');
    }
    uninstall() {
        this.listeners.forEach(({ target, event, callback }) => {
            target.removeEventListener(event, callback);
            this.status[event] = false;
        });
    }
}
