export const _log = console.log.bind(null, '[Recorder]:');
export const _error = console.error.bind(null, '[Recorder]:');
export const _warn = console.warn.bind(null, '[Recorder]:');
export function _now() {
    if (!window.performance)
        return Date.now();
    return Math.floor(performance.now());
}
export function _throttle(func, wait = 100) {
    let previous = _now();
    return function (...args) {
        const now = _now();
        const restTime = now - previous;
        if (restTime >= wait) {
            previous = now;
            return func.apply(this, args);
        }
    };
}
export function _newuuid() {
    return Math.random()
        .toString(16)
        .split('.')[1];
}
export function _replace(source, name, replacement) {
    const original = source[name];
    function doReplace() {
        const wrapped = replacement(original);
        wrapped.__recorder__ = true;
        wrapped.__recorder_original__ = original;
        source[name] = wrapped;
    }
    if (original) {
        if (!(name in source) || original.__recorder__)
            return;
        doReplace();
        return;
    }
    else if (original === null || original === undefined) {
        doReplace();
        return;
    }
}
export function _recover(source, name) {
    if (!(name in source) || !source[name].__recorder__)
        return;
    const { __recorder_original__ } = source[name];
    source[name] = __recorder_original__;
}
export function _parseURL(href = location.href) {
    const match = href.match(/^(([^:\/?#]+):)?(\/\/([^\/?#]*))?([^?#]*)(\?([^#]*))?(#(.*))?$/);
    if (!match)
        return {};
    const query = match[6] || '';
    const fragment = match[8] || '';
    return {
        protocol: match[2],
        host: match[4],
        path: match[5],
        query,
        fragment,
        relative: match[5] + query + fragment
    };
}
export function _seralize(obj) {
    return Object.keys(obj)
        .map(k => `${k}=${obj[k]}`)
        .join('&');
}
